////////////////////////////////////////////////////
#ifndef __P2TPROCESSOR_H_
#define __P2TPROCESSOR_H_

class P2TProcessor;

#define PDF_OUTPUTPROCESSDETAIL 0x01
#define PDF_OUTPUTPROCESSTIME   0x10
#define PDF_OUTPUTRANGE         0x20
#define PDF_OUTPUTPDFINFO       0x40

/**
 * This is the log infromation callback function. if you wanna the real time
 * log/output information, you need hook up this function.
 * @param char* i_szStr The input log information.
 *
 */
typedef void(*LOGCALLBACKFUNC)(char* i_szStr);

/**
 * This is the log infromation callback function with context.
 * @param char* i_szStr The input log information.
 * @param void* i_pContext
 *
 */
typedef void(*LOGCALLBACKFUNC1)(char* i_szStr, void* i_pContext);

/**
 * This is the data stream callback function. you will get the stream data here.
 * Add your own process logic here to process the output data stream.
 * @param const char* i_szStr  The buffer pointer of the output string
 * @param int   i_size   The size of the data(in byte)
 */
typedef void(*DATACALLBACKFUNC)(const char* i_szStr, int i_size);

/**
 * This is the data stream callback function with context pointer. 
 * you will get the stream data here.
 * Add your own process logic here to process the output data stream.
 * @param const char* i_szStr  The buffer pointer of the output string
 * @param int   i_size     The size of the data(in byte)
 * @param void* i_pContext The context pointer
 */
typedef void(*DATACALLBACKFUNC1)(const char* i_szStr, int i_size, void* i_pContext);

/**
 * This is the one of two methods exposed from the DLL
 * Use this method to create a new instance of P2TProcessor
 * and return the pointer of the processor, The pointer must
 * be released use ReleaseProcessor after converting.
 */
typedef P2TProcessor* (*CREATEPROCESSOR)();
 
/**
 * This is the one of the two methods exposed from the DLL
 * Use this method to release the processor instance created
 * by CreateProcessor
 *
 */
typedef void (*RELEASEPROCESSOR)(P2TProcessor* pProcessor);

/**
 * P2TProcessor is the class that encapsulates all the functions related
 * to PDF converting. You can get the instance of this class through 
 * CreateProcessor exposed from the DLL
 *
 */
class P2TProcessor {
public:

/**
 * This is the function that should be called first before you try to call
 * any other functions, It verifies the run time license information.
 * @param char* UserName  The register username.
 * @param char* Key       The register keycode.
 * @return int   zero:    Success. In trial version, always return zero.
 *           not zero:    Something wrong with username/key
 */
virtual int VerifyLicense(char* UserName, char* Key) = 0;
 
/**
 * EngageProcessor is a helper method. It sets the page range /page breaker
 * and other control flag for the following converting process. This function
 * Don't need to be called if you just want output the whole pdf file to text
 * file without any special control.
 *
 * @param int i_CtrlFlag: The contrl flags you can pass in to control the
 * behavior of the PDF Processor.
 * The CtrlFlag can be the following value or their's combination
 * PDF_OUTPUTPROCESSDETAIL 0x01
 * PDF_OUTPUTPROCESSTIME   0x10
 * PDF_OUTPUTRANGE         0x20
 * PDF_OUTPUTPDFINFO       0x40
 *
 * PDF_OUTPUTPROCESSDETAIL: Output the detail information of processing
 * The output string will be return by pbstrRetVal
 * in Convert method, See Convert for detail.
 *
 * PDF_OUTPUTPROCESSTIME: Output the process time for every pdf file.
 * The output string will be return by pbstrRetVal
 * in Convert method, See Convert for detail.
 *
 * PDF_OUTPUTRANGE: Knowledge the processor only a subset of pages
 * need to be output, the page range pass in through
 * i_bstrPageRange. see i_bstrPageRange.
 * 
 * PDF_OUTPUTPDFINFO: Knowledge the processor to output the document's
 * information of current pdf to the beginning of the
 * output text file.
 *
 * @param char* i_bstrPageRange: The PageRange string to identify which pages
 * should be output example: "1,5,2-18,20,23"
 * This paramter will be check if PDF_OUTPUTRANGE flag was set.
 * 
 * @param char* i_bstrPageBreaker: The string that seperate each page.
 * example: "~~~~~~~~~~~~~~~~~~My Document~~~~~Page: %%NUM~~~~~~~~~~~~~~~~~"
 *
 */
virtual void EngagePDFProcessor(LOGCALLBACKFUNC pf, int i_nCntrlFlg, char* i_szPageRange, char* i_szPageBreaker) = 0;

/**
 * This is the helper function with context as pass in parameters
 * @param LOGCALLBACKFUNC1 pf log call back function with context
 * @param void* i_pContext The context
 */
virtual void EngagePDFProcessor(LOGCALLBACKFUNC1 pf, int i_nCntrlFlg, char* i_szPageRange, char* i_szPageBreaker, void* i_pContext) = 0;
 
/**
 * This function parse pdf file and output the data into a file
 * @param char* i_pdfFile   The input pdf file's path and name.
 * @param char* i_txtFile   The output text file's path and name.
 * @return int    0   Success.
 *               -1   Something wrong during converting, check log callback func
 */
virtual int ParsePDF(char* i_pdfFile, char* i_txtFile) = 0;

/**
 * This function parse pdf file and output the data stream into the callback
 * function 
 * @param char* i_szPDFName:           The input pdf file's path and name.
 * @param DATACALLBACKFUNC i_pFunc:    The pointer of the callback function 
 * @param int i_nBufSize:              The max buffer size that will pass to the
 *                                     callback function.
 * @return int    0  Success.
 *               -1  Something wrong during converting, check log callback func 
 */
virtual int ParsePDF(const char* i_szPDFName, DATACALLBACKFUNC i_pFunc, int i_nBufSize) = 0;

/**
 * This function parse pdf file and output the data stream into the callback
 * function 
 * @param char* i_szPDFName:           The input pdf file's path and name.
 * @param DATACALLBACKFUNC i_pFunc:    The pointer of the callback function, context is passed 
 *                                     in EngagePDFProcessor
 * @param int i_nBufSize:              The max buffer size that will pass to the
 *                                     callback function.
 * @return int    0  Success.
 *               -1  Something wrong during converting, check log callback func 
 */
virtual int ParsePDF(const char* i_szPDFName, DATACALLBACKFUNC1 i_pFunc, int i_nBufSize) = 0;


/**
 * Get current version information(build number of parsing engine).
 */
virtual const char* GetVersion() = 0;

};

#endif 